/*
 * This file is part of the LinuxBIOS project.
 *
 * Copyright (C) 2004 Tyan Computer
 * Written by Yinghai Lu <yhlu@tyan.com> for Tyan Computer.
 * Copyright (C) 2006,2007 AMD
 * Written by Yinghai Lu <yinghai.lu@amd.com> for AMD.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */

#include <console/console.h>
#include <device/device.h>
#include <device/pci.h>
#include <device/pci_ids.h>
#include <device/pci_ops.h>
#include <arch/io.h>
#include "sis966.h"
#if CONFIG_USBDEBUG_DIRECT
#include <usbdebug_direct.h>
#endif

extern struct ehci_debug_info dbg_info;

// From Y.S.
// PCI R43h-R40h=00000020h
// PCI R4Bh-R48h=00078010h
uint8_t	SiS_SiS7002_init[15][3]={
{0x74, 0x00, 0x00},
{0x75, 0x00, 0x00},
{0x76, 0x00, 0x00},
{0x77, 0x00, 0x00},
{0x04, 0x00, 0x00},
{0x40, 0x00, 0x20},
{0x41, 0x00, 0x00},
{0x42, 0x00, 0x00},
{0x43, 0x00, 0x00},
{0x44, 0x00, 0x64},
{0x48, 0x00, 0x10},
{0x49, 0x00, 0x80},
{0x4A, 0x00, 0x07},
{0x4B, 0x00, 0x00},
{0x00, 0x00, 0x00}					//End of table
};



static void usb2_init(struct device *dev)
{
       uint8_t *base;
	struct resource *res;
	uint32_t temp32;


	printk_debug("usb2_init\r\n");
//-------------- enable USB2.0 (SiS7002) -------------------------
{
       uint8_t  temp8;
       int i=0;

printk_debug("usb_init(2.0)\n");

	while(SiS_SiS7002_init[i][0] != 0)
	{				temp8 = pci_read_config8(dev, SiS_SiS7002_init[i][0]);
					temp8 &= SiS_SiS7002_init[i][1];
					temp8 |= SiS_SiS7002_init[i][2];
					pci_write_config8(dev, SiS_SiS7002_init[i][0], temp8);
					i++;
	};
}
res = find_resource(dev, 0x10);
	if(!res)
		return;

	base =(uint8_t *) res->base;
	printk_debug("base = %08x\n", base);
       writel(0x2,base+0x20);
//-----------------------------------------------------------

#if 0
{
//device_t dev;
int i;
//dev = pci_locate_device(PCI_ID(0x1039, 0x5513), 0);
printk_debug("USB 2.0 PCI config \n");
for(i=0;i<0xFF;i+=4)
{    if((i%16)==0)
    {print_debug("\r\n");print_debug_hex8(i);print_debug("  ");}
    print_debug_hex32(pci_read_config32(dev,i));
    print_debug("  ");
}

}

#endif



}

static void usb2_set_resources(struct device *dev)
{
#if CONFIG_USBDEBUG_DIRECT
	struct resource *res;
	unsigned base;
	unsigned old_debug;

	old_debug = get_ehci_debug();
	set_ehci_debug(0);
#endif
	pci_dev_set_resources(dev);

#if CONFIG_USBDEBUG_DIRECT
	res = find_resource(dev, 0x10);
	set_ehci_debug(old_debug);
	if (!res) return;
	base = res->base;
	set_ehci_base(base);
	report_resource_stored(dev, res, "");
#endif

}

static void lpci_set_subsystem(device_t dev, unsigned vendor, unsigned device)
{
	pci_write_config32(dev, 0x40,
		((device & 0xffff) << 16) | (vendor & 0xffff));
}
static struct pci_operations lops_pci = {
	.set_subsystem	= lpci_set_subsystem,
};

static struct device_operations usb2_ops  = {
	.read_resources	= pci_dev_read_resources,
	.set_resources	= usb2_set_resources,
	.enable_resources	= pci_dev_enable_resources,
	.init		= usb2_init,
//	.enable		= sis966_enable,
	.scan_bus	= 0,
	.ops_pci	= &lops_pci,
};

static struct pci_driver usb2_driver __pci_driver = {
	.ops	= &usb2_ops,
	.vendor	= PCI_VENDOR_ID_SIS,
	.device	= PCI_DEVICE_ID_SIS_SIS966_EHCI,
};
